@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import
 import os.path as osp
-import logging, subprocess, os, glob, shutil, time
+import logging, subprocess, os, glob, shutil, time, datetime
 from contextlib import contextmanager
 
 DEFAULT_LOGGING_LEVEL = logging.WARNING
@@ -35,6 +35,16 @@ def drymode(flag=True):
     global DRYMODE
     DRYMODE = flag
 
+@contextmanager
+def drymode_context(flag=True):
+    global DRYMODE
+    _saved_DRYMODE = DRYMODE
+    DRYMODE = flag
+    try:
+        yield DRYMODE
+    finally:
+        DRYMODE = _saved_DRYMODE
+
 def is_string(string):
     """
     Checks strictly whether `string` is a string
@@ -64,7 +74,7 @@ def run_command(cmd, dry=None, non_zero_exitcode_ok=False, n_retries=0):
     i_attempt = 0
     if dry is None: dry = DRYMODE
     while True:
-        logger.info('Issuing command (attempt %s: %s)', i_attempt, ' '.join(cmd))
+        logger.info('%sIssuing command (attempt %s: %s)', '(dry) ' if dry else '', i_attempt, ' '.join(cmd))
         if dry: return ''
         process = subprocess.Popen(
             cmd,
@@ -85,7 +95,7 @@ def run_command(cmd, dry=None, non_zero_exitcode_ok=False, n_retries=0):
             logger.info('Command exited with status 0 - all good')
         else:
             if non_zero_exitcode_ok:
-                logger.info('Command exited with status %s', return_code)
+                logger.info('Command exited with status %s', returncode)
                 return returncode
             else:
                 logger.error('Exit status {0} for command: {1}'.format(returncode, cmd))
@@ -248,6 +258,7 @@ def _join_mgm_lfn(mgm, lfn):
             'This function expects filenames that start with \'/\''
             )
     if not mgm.endswith('/'): mgm += '/'
+    # logger.error('mgm=%s lfn=%s', mgm, lfn)
     return mgm + lfn
 
 def format(path, mgm=None):
@@ -317,6 +328,7 @@ def read_cache(subcache_name, key):
     if not(val is None): logger.debug('Using cached result for %s from cache %s', key, subcache_name)
     return val
 
+_LAST_CACHE_WRITE = None
 def write_cache(subcache_name, key, value):
     """
     Writes a value to a cache
@@ -326,23 +338,44 @@ def write_cache(subcache_name, key, value):
         subcache = make_cache(subcache_name)
         subcache[key] = value
         subcache.sync()
+        global _LAST_CACHE_WRITE
+        _LAST_CACHE_WRITE = datetime.datetime.now()
 
-def tarball_cache(dst='seutils-cache.tar.gz'):
+_LAST_TARBALL_CACHE = None
+_LAST_TARBALL_PATH = None
+def tarball_cache(dst='seutils-cache.tar.gz', only_if_updated=False):
     """
-    Dumps the cache to a tarball
+    Dumps the cache to a tarball.
+    If only_if_updated is True, an additional check is made to see whether
+    the last call to tarball_cache() was made after the last call to write_cache();
+    if so, the last created tarball presumably still reflects the current state of
+    the cache, and no new tarball is created. This will only work within the same python
+    session (timestamps are not saved to files).
     """
+    global _LAST_TARBALL_CACHE, _LAST_TARBALL_PATH
     if not USE_CACHE: raise Exception('No active cache to save to a file')
     if not dst.endswith('.tar.gz'): dst += '.tar.gz'
     dst = osp.abspath(dst)
+    if only_if_updated:
+        if _LAST_TARBALL_CACHE:
+            if _LAST_CACHE_WRITE is None or _LAST_CACHE_WRITE < _LAST_TARBALL_CACHE:
+                # Either no write has taken place or it was before the last tarball creation;
+                # use the last created tarball and don't run again
+                logger.info('Detected no change w.r.t. last tarball %s; using it instead', _LAST_TARBALL_PATH)
+                return _LAST_TARBALL_PATH
     try:
         _return_dir = os.getcwd()
+        if not osp.isdir(CACHEDIR): os.makedirs(CACHEDIR) # Empty dir can be tarballed too for consistency
         os.chdir(CACHEDIR)
         cmd = ['tar', '-zcvf', dst, '.']
         logger.info('Dumping %s --> %s', CACHEDIR, dst)
         run_command(cmd)
+        _LAST_TARBALL_CACHE = datetime.datetime.now()
+        _LAST_TARBALL_PATH = dst
         return dst
     finally:
         os.chdir(_return_dir)
+    return dst
 
 def load_tarball_cache(tarball, dst=None):
     """
@@ -507,7 +540,8 @@ def stat_function(*args, **kwargs):
 
 def _stat_gfal(path, not_exist_ok=False):
     import datetime
-    output = run_command(['gfal-stat', path], non_zero_exitcode_ok=not_exist_ok)
+    cmd = ['gfal-stat', path]
+    output = run_command(cmd, non_zero_exitcode_ok=not_exist_ok)
     if isinstance(output, int):
         # The command failed; if output is 2 the path did not exist,
         # which might be okay if not_exist_ok is True, but other codes
@@ -998,6 +1032,7 @@ def cli_detect_fnal():
         set_default_mgm(mgm)
 
 def cli_flexible_format(lfn, mgm=None):
+    cli_detect_fnal()
     if not has_protocol(lfn) and not lfn.startswith('/'):
         try:
             prefix = '/store/user/' + os.environ['USER']
